package fzj.matse.tictactoe;

import java.util.ArrayList;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.View;
import android.view.*;
import android.support.constraint.ConstraintLayout.LayoutParams;

/**
 * custom view: Spielfeld fuer TicTacToe
 */
public class TicTacToeView extends View
{
	private static final int strokew=40;
	private PlayingField spielfeld;
	private Paint paint = new Paint();
	private ArrayList<UserSelectionListener> listener = new ArrayList<UserSelectionListener>();

	public TicTacToeView(Context context, AttributeSet attrs) {
		super(context, attrs);
		setFocusable(true);
		setFocusableInTouchMode(true);
		this.setOnTouchListener( (v,event) -> onTouch(event) );
	}

	/** Diese Methode wird nur benoetigt, wenn die View nur ein Teil des Bildschirms ist. Dann muss die gewuenschte Groesse angegeben werden */
	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec)
	{
		int width = MeasureSpec.getSize(widthMeasureSpec);
		int height = MeasureSpec.getSize(heightMeasureSpec);

		int max = Math.min( width, height );
		setMeasuredDimension( max, max );
	}

	public void setData(PlayingField s) {
		spielfeld = s;
	}

	public void addSelectionListener(UserSelectionListener l) {
		listener.add(l);
	}
	public void updateView() {
		System.out.println("updateView wurde aufgerufen");
		postInvalidate(); // ruft onDraw indirekt auf
	}

	@Override
	protected void onDraw(Canvas canvas)
	{
		// Gelben Hintergrund zeichnen
		paint.setStyle(Paint.Style.FILL);
		paint.setColor(Color.YELLOW);
		canvas.drawRect(0, 0, getWidth(), getHeight(), paint);

		// gruenes Gitternetz zeichnen
		paint.setStrokeWidth(strokew);
		paint.setStyle(Paint.Style.STROKE);
		paint.setColor(Color.GREEN);
		int width = getWidth();
		int s=strokew/2;
		canvas.drawRect(s, s,              width-s, width/3, paint);
		canvas.drawRect(s, width*2/3, width-s, width-s, paint);
		canvas.drawRect(s, s, width/3,width-s, paint);
		canvas.drawRect(width*2/3, s,  width-s, width-s, paint);


		int max = Math.min(  getWidth(), getHeight() );
		int fieldWidth = (max - 4*strokew) / 3 * 7/10;
		// Kreise fuer die Spielsteine zeichnen
		if (spielfeld != null) {
			for (int i=0; i<spielfeld.size(); i++) {
				for (int j=0; j<spielfeld.size(); j++) {
					int value = spielfeld.value(i, j);
					if (value != 0) {
						int len = (width-strokew)/3;
						int x = (int) ((i+0.5)*len) + strokew/2;
						int y = (int) ((j+0.5)*len) + strokew/2;
						if (value == 1) {
							paint.setColor(Color.BLUE);
							canvas.drawCircle(x, y, fieldWidth/2, paint);
						}
						else {
							paint.setColor(Color.RED);
							canvas.drawCircle(x, y, fieldWidth/2, paint);
						}
					}
				}
			}
		}
	}


	/**
	 * wird von onTouch aufgerufen, nachdem ein Element vom Benutzer angeklickt wure
	 * @param x
	 * @param y
	 */
	private void elementSelected(int x, int y) {
		for (UserSelectionListener l : listener) {
			l.selected(x, y);
		}
	}

	/**
	 * wird aufgerufen, wenn der Benutzer den Bildschirm berührt
	 */
	public boolean onTouch(MotionEvent event)
	{
		// wird ausgeloest nach Berühren und Loslassen des Bildschirms
		System.out.println("touch " + event.getX() +" "+ event.getY());
		int elemSize = getHeight() / 3;
		int x = (int) event.getX()/elemSize;
		int y = (int) event.getY()/elemSize;

		elementSelected(x,y);
		return true;
	}

}
